//       THIS FILE STARTS THE GENETIC ALGORITHM FOR OBSERVER'S GAINS SELECTION
//
//                     Created by Tadeusz Białoń 
//                      tadeusz.bialon@polsl.pl
//
//           Created and tested in Scilab 5.4.1.
//           Other versions of Scilab may require some modifications
//           to be introduced in this code.
//           Scilab is an open source software for numerical computation,
//           avaliable at www.scilab.org. 
//
// This optimization algoritm is free for scientific and educational purposes.
// If You use it in your research, please cite the Energies MDPI paper:
//    Tadeusz Białoń, Marian Pasko, Roman Niestrój, 'Developing induction motor  
//    state observers with increased robustness'
//
//   You can change the parameters of the motor, genetic algoritm and fitness  
//   function below, in section 'PARAMETERS'. You do not need to modify the
//   contents of 'OPTIMIZATION' and 'GRAPHS' sections, as well as the contents
//   of file 'genet functions.sci', but you can watch them for reference.
//
//   To start the optimization:
//   1. Set the path to this file as a working directory. To do that type 
//      in Scilab console:
//      --> cd('C:\directory\subdirectory')
//   4. Ensure that the working directory contains also the file 
//      'genet functions.sci'.
//   3. Execute this file. To do that type:
//      --> exec('START genet.sce',-1)
//   4. Once the optimization is successfull, the graph window will appear.
//      Calculated gain matrices of the observer are displayed in the Scilab 
//      console window.
//
//   Remember that the gain selection process is based on random numbers. 
//   Therefore, the algorithm launched several times with the same settings may 
//   return different result each time.





clear();

// PATRAMETERS ----------------------------------------------------------------

// induction motor nominal parameters:
Un=400; Pn=7500; In=14.6; nn=1450; // [V,W,A,rpm]
Rs=0.56; Rr=0.72; Lm=0.1183; Lsigma=0.0043; Ls=Lm+Lsigma; Lr=Ls;// [Ohm,H]
Mn=Pn*30/(%pi*nn); // nominal torque [Nm]
J=0.6; // moment of inertia

// base quantities for per-unit system (p.u.):
Uodn=Un; Iodn=In*sqrt(3); omegaodn=2*%pi*50; pb=2;
Zodn=Uodn/Iodn; Lodn=Uodn/(omegaodn*Iodn); Yodn=Uodn/omegaodn;
Modn=Uodn*Iodn*pb/omegaodn; Jodn=Uodn*Iodn*pb/(omegaodn^3); todn=1/omegaodn;

// induction motor p.u. parameters:
Rsr=Rs/Zodn; Rrr=Rr/Zodn; Lsr=Ls/Lodn; Lrr=Lr/Lodn; Lmr=Lm/Lodn;
Jr=J/Jodn; Mnr=Mn/Modn;

// matrices of the mathematical model (1), (3), (4) of the motor:
a=Lmr/(Lmr^2-Lsr*Lrr); b=Lsr/(Lmr^2-Lsr*Lrr); c=Lrr/(Lmr^2-Lsr*Lrr); 
I2=eye(2,2); J2=[0, -1; 1, 0];
A=[Rsr*c*I2, -Rsr*a*I2; -Rrr*a*I2, Rrr*b*I2];
A3=[zeros(2,2), zeros(2,2); zeros(2,2), J2];
B=[I2; zeros(2,2)];
C=[-c*I2, a*I2];

// matrices of the proportional observer:
Ao=A;   // TIPS: 1. If You want to calculate the gains of an observer different 
A3o=A3; //          than the proportional one, replace the matrices in lines 70 - 73
Bo=B;   //          with proper forms from section 3.3. For reference see
Co1=C;  //          the file 'START sim.sce', lines 67 - 226.
        //       2. For different types of observers settings of the fitness 
        //          function must be adjusted. It comes from different dynamical
        //          properties of non-proportional observers.
        //       3. The more complicated structure of the observer, the greater
        //          possibility that the gain selection process will not return
        //          satisfying result. 

// angular speed values for which the values of the fitnees function (20) are to
// be calculated (red dots at resulting graphs):
omega=[0 0.05 0.1 0.2 0.4 0.6 0.8 1 1.2]; // [p.u.]
                                          // the vector must contain at last one element

// structure of the gain matrices:
Kschem=[1, 0;..    // TIPS: 1. Kschem and K3schem determine the forms of gain
        0, 1;..    //          matrices K and K3. The forms should be consistent
        2, 0;..    //          with (5) to ensure symmetry of the observer (2nd
        0, 2];     //          criterion in point 3.2.3). Example:
K3schem=[0, 3;..   //          Kschem=[1, 0;..     K3schem=[0, 3;..
        -3, 0;     //                  0, 1;..             -3, 0;
         0, 0;     //                  2, 0;..              0, 0;
         0, 0];    //                  0, 2];               0, 0];
                   //          means that there are 3 gains to be optimized, a, b 
                   //          and c and resulting gain matrix has the form:
                   //                             | a  0 |           | 0  c |
                   //         Ko = K + omega*K3 = | 0  a | + omega * |-c  0 |
                   //                             | b  0 |           | 0  0 |
                   //                             | 0  b |           | 0  0 |
                   //       2. Matrices Kschem and K3schem have two columns and
                   //          the same number of rows as the matrix Ao 

// search space limits:
RangeMac=[-0.5, 0.5;    // TIPS: 1. RangeMac determines the ranges of values of 
          -0.3, 0.3;    //          optimized gains. Example:
          -0.1, 0.1];   //          RangeMac=[-0.5, 0.5;
                        //                    -0.3, 0.3;
                        //                    -0.1, 0.1];
                        //          means that gain a belongs to the range from 
                        //          -0.5 to 0.5, gain b belongs to the range from 
                        //          -0.3 to 0.3 and gain c belongs to the range
                        //          from -0.1 to 0.1.
                        //       2. RangeMac must have the same number of rows 
                        //          as the number of optimized gains, determined by 
                        //          Kschem and K3schem.
                        //       3. If the ranges are too narrow, it may happen that
                        //          obtained eigenvalues of the observer will be 
                        //          very different from desired ones.
                        //          If the range limit values are too great, it may 
                        //          happen that the resulting amplification index 
                        //          value (18) will be unfavourably high.

// genetic algorithm settings:
sel.cross='av';     // crossing-over method:
                    //      'av' - proportional, example:
                    //             gen1 = [x1 x2 x3] gen2 = [y1 y2 y3]
                    //             result = [r*x1+(r-1)*y1 r*x2+(r-1)*y2 r*x3+(r-1)*y3]
                    //             r - random number from 0 to 1
                    //      '1p' - one-point, example:
                    //             result = [x1 y2 y3] or result = [x1 x2 y3]
                    //      '2p' - two-point, example:
                    //              result = [x1 y2 x3]
sel.typ='turn';     // selection method:
                    //      'elit' - elitism, a percent of the best chromosomes in each
                    //               generation are to be crossed
                    //      'rulet' - roulette method
                    //      'turn' - tournament method, two groups of chromosomes are randomly 
                    //               chosen then two best chromosomes from each group are crossed
sel.proc=4;         // percentage:
                    //      if sel.typ = 'elit' - percent of chromosomes in each generation
                    //                            taken for crossing
                    //      if sel.typ = 'rulet' - ignored
                    //      if sel.typ = 'turn' - the percent size of the tournament group
                    //              in reference to the total number of chromosomes in the
                    //              generation Nchr
Nchr=500;           // number of chromosomes in each generation
Npok=25;            // number of generations
pmut=2;             // mutation probability [%]

// fitness function (20) parameters, refer to the table 1.
// CAUTION: In the structure 'wagi' below the numbers of components are different than 
//          in the paper. See the comments below for numbers corresponding to the paper.

// component F1 described with formula (21):
wagi.f1.w=20; // weight coefficient w_1

// component F3 described with formula (23):
wagi.f2.w=1; // weight coefficient w_3
wagi.f2.lzad=[-2];   // 'lambda_ref' polynomial, see formula (24)
                       // if the vector has 1 element than:
                       //    labda_ref(omega) = wagi.f3.lzad(1) 
                       // if the vector has 3 elements than:
                       //    labda_ref(omega) = wagi.f3.lzad(1) +..
                       //                     + wagi.f3.lzad(2) * omega^2 +..
                       //                     + wagi.f3.lzad(3) * omega^4 

// component F4 described with formula (25):
wagi.f3.w=1; // weight coefficient w_4
wagi.f3.lzad=[-0.96, -0.96, 0.32]; // 'lambda_ref' polynomial, see formula (24)

// component F7 described with formula (28):
wagi.f4.w=0.05; // weight coefficient w_7

// component F9 described with formula (30):
wagi.f5.w=0.2; // weight coefficient w_9

// component F2 described with formula (22):
wagi.f6.w=1; // weight coefficient w_2

// component F5 described with formula (26):
wagi.f7.w=1; // weight coefficient w_5
wagi.f7.lzad=[-0.195, -0.065, -0.0325]; // 'lambda_ref' polynomial, see formula (24)

// component F6 described with formula (27):
wagi.f8.w=0.1; // weight coefficient w_6
wagi.f8.lzad=[-2.6, 0.65, -0.325]; // 'lambda_ref' polynomial, see formula (24)

// component F8 described with formula (29):
wagi.f9.w=0.1; // weight coefficient w_8
wagi.f9.lzad=[0.3, 0.9, -0.3]; // 'lambda_ref' polynomial, see formula (24)




// OPTIMIZATION ---------------------------------------------------------------

exec('genet functions.sci',-1); // activate the genetic algorithm functions
SetRandGen(); // restart the random number generator

// run the genetic algorithm:
[K,K3,data]=Genet(Ao,A3o,Co1,Kschem,K3schem,RangeMac,omega,wagi,sel,pmut,Nchr,Npok);

// display the results in the Scilab console:
format(10);
disp('Optimization result:'); 
disp('    K = ');disp(K); 
disp('    K3 = ');disp(K3);




// GRAPHS ---------------------------------------------------------------------
NpktWkr=200; // number of points for each curve in the graph

MainWindow=figure(1);  clf(MainWindow); 
set(MainWindow,"color_map", jetcolormap(64));
MainWindow.auto_resize='on';
MainWindow.background=-2;
MainWindow.figure_size=[600,800];
MainWindow.figure_position=[10,10];

// calculation of eigenvalues "lambda" and amplification index "mu"
for ii=NpktWkr:-1:1 do
    om(ii)=-1.2+2.4/(NpktWkr-1)*(ii-1);
    lambda=spec(Ao+A3o*om(ii)+..
    (K+K3*om(ii))*Co1);
    tmplre=gsort(real(lambda));
    tmplim=gsort(imag(lambda));
    for jj=size(tmplre,'*'):-1:1 do
        lre(ii,jj)=tmplre(jj);
        lim(ii,jj)=tmplim(jj);
        lzespolone(ii,jj)=lambda(jj)
    end
    wsk(ii)=WskWzm(K+K3*om(ii));
end

// Re(lambda) graph
wlre=newaxes();
tmplre=[%nan]; tmpom=[%nan]; tmplim=[%nan];
for ii=size(Ao,'r'):-1:1 do
    tmplre=[tmplre;lre(:,ii);%nan];
    tmplim=[tmplim;lim(:,ii);%nan];
    tmpom=[tmpom;om;%nan];
end
sca(wlre); plot(tmpom,tmplre,'b',..
omega,data.ParGenet.omega*0,'r.');
wlre.axes_visible=['on' 'on' 'on'];
wlre.background=-2;
wlre.margins=[0.1 0.05 0.05 0.05]; 
wlre.axes_bounds=[0 0 1 0.3]; 
wlre.y_label.text='${\rm Re}(\lambda)$';
wlre.y_label.font_size=3.5;
wlre.grid=[1 1 1];
wlre.children.children.thickness=2;
wlre.x_label.text='$\omega \phantom{x} \rm {p.u.}$';
wlre.x_label.font_size=4;

// Im(lambda) graph
wlim=newaxes();
sca(wlim); plot(tmpom,tmplim,'b',..
data.ParGenet.omega,data.ParGenet.omega*0,'r.');
wlim.axes_visible=['on' 'on' 'on'];
wlim.background=-2;
wlim.margins=[0.1 0.05 0.05 0.05]; 
wlim.axes_bounds=[0 0.35 1 0.3]; 
wlim.y_label.text='${\rm Im}(\lambda)$';
wlim.y_label.font_size=3.5;
wlim.grid=[1 1 1];
wlim.children.children.thickness=2;
wlim.x_label.text='$\omega \phantom{x} \rm {p.u.}$';
wlim.x_label.font_size=4;

// mu graph
wwsk=newaxes();
sca(wwsk); plot(om,wsk);
wwsk.axes_visible=['on' 'on' 'on'];
wwsk.background=-2;
wwsk.margins=[0.1 0.05 0.1 0.2]; 
wwsk.axes_bounds=[0 0.7 1 0.3]; 
wwsk.y_label.text='$\mu$';
wwsk.y_label.font_size=3.5;
wwsk.x_label.text='$\omega \phantom{x} \rm {p.u.}$';
wwsk.x_label.font_size=4;
wwsk.grid=[1 1 1];
wwsk.children.children.thickness=2;

