//       THIS FILE STARTS THE SIMULATION OF THE CHOSEN OBSERVER
//
//                     Created by Tadeusz Białoń 
//                      tadeusz.bialon@polsl.pl
//
//           Created and tested in Scilab 5.4.1.
//           Other versions of Scilab may require some modifications
//           to be introduced in this code.
//           Scilab is an open source software for numerical computation,
//           avaliable at www.scilab.org. 
//
// This simulation model is free for scientific and educational purposes.
// If You use this model in your research, please cite the Energies MDPI paper:
//    Tadeusz Białoń, Marian Pasko, Roman Niestrój, 'Developing induction motor  
//    state observers with increased robustness'
//
//   You can chose the observer type below in the section 'PARAMETERS'(line 68). 
//   You can also change the parameters of the simulation there. You do not need 
//   to modify the contents of 'SIMULATION' and 'GRAPHS' sections, as well as 
//   the contents of files 'sim functions.sci' and 'sim model.zcos', but you can 
//   watch them for reference.
//
//   To start the simulation:
//   1. Set the path to this file as a working directory. To do that type 
//      in Scilab console:
//      --> cd('C:\directory\subdirectory')
//   4. Ensure that the working directory contains also the files 
//      'sim functions.sci' and 'sim model.zcos'.
//   3. Execute this file. To do that type:
//      --> exec('START sim.sce',-1)
//   4. Once the simulation is successfull, the graph window will appear.
//      It may take several minutes. Watch the Scilab console for progress prompts.




clear();

// PARAMETERS -----------------------------------------------------------------

// induction motor nominal parameters:
Un=400; Pn=7500; In=14.6; nn=1450; // [V,W,A,rpm]
Rs=0.56; Rr=0.72; Lm=0.1183; Lsigma=0.0043; Ls=Lm+Lsigma; Lr=Ls;// [Ohm,H]
Mn=Pn*30/(%pi*nn); // nominal torque [Nm]
J=0.6; // moment of inertia
// CAUTION: Defined below gain matrices of the observers are obtained for this
//          particular set of nominal paramaters. Once You changed them, You 
//          should calculate new gains values!

// base quantities for per-unit system (p.u.):
Uodn=Un; Iodn=In*sqrt(3); omegaodn=2*%pi*50; pb=2;
Zodn=Uodn/Iodn; Lodn=Uodn/(omegaodn*Iodn); Yodn=Uodn/omegaodn;
Modn=Uodn*Iodn*pb/omegaodn; Jodn=Uodn*Iodn*pb/(omegaodn^3); todn=1/omegaodn;

// induction motor p.u. parameters:
Rsr=Rs/Zodn; Rrr=Rr/Zodn; Lsr=Ls/Lodn; Lrr=Lr/Lodn; Lmr=Lm/Lodn;
Jr=J/Jodn; Mnr=Mn/Modn;

// matrices of the mathematical model (1), (3), (4) of the motor:
a=Lmr/(Lmr^2-Lsr*Lrr); b=Lsr/(Lmr^2-Lsr*Lrr); c=Lrr/(Lmr^2-Lsr*Lrr); 
I2=eye(2,2); J2=[0, -1; 1, 0];
A=[Rsr*c*I2, -Rsr*a*I2; -Rrr*a*I2, Rrr*b*I2];
A3=[zeros(2,2), zeros(2,2); zeros(2,2), J2];
B=[I2; zeros(2,2)];
C=[-c*I2, a*I2];

// Selection of the observer: 
obsrvr=1;
//   1 - proportional (P) observer (section 3.2, fig. 5)
//   2 - proportional-integral (PI) observer (section 3,3,1, fig. 6)
//   3 - reduced order integral unit PI observer (section 3.3.2, fig. 7)
//   4 - modified integral observer (section 3.3.3, fig. 8)
//   51 - proportional observer with 1 additional integrator (section 3.3.4, fig. 9)
//   52 - proportional observer with 2 additional integrators (section 3.3.4, fig. 10)
select obsrvr
case 1 then // proportional observer matrices
    Ao=A; A3o=A3; Co1=C; Bo=B; 
    // Observer's gain matrices Ko = K + omega*K3 :
    K = [- 0.2017193,   0;..
           0,         - 0.2017193;..
           0.0498614,   0;..
           0,           0.0498614];
    K3= [  0,           0;..
           0,           0;..
           0,           0.0840620;..
         - 0.0840620,   0];
    // speed adaptation mechanism gains:
    okp=0.001; // proportional unit gain (p.u.)
    oki=0.1; // integral unit gain (p.u.)
case 2 then // PI observer matrices, formulas (33) - (34)
    Ao=[A, eye(4,4); zeros(4,4), -diag([0.1 0.1 0.1 0.1])];
    A3o= [A3, zeros(4,4); zeros(4,8)];
    Bo=[B; zeros(4,2)];
    Co1=[C, zeros(2,4)];
    // Observer's gain matrices Ko = K + omega*K3 :
    K = [- 0.1139844,   0;..
           0,         - 0.1139844;..
           0.0336825,   0;..
           0,           0.0336825;..
         - 0.0428698,   0;..
           0,         - 0.0428698;..
           0.0547550,   0;..
           0,           0.0547550];
    K3= [  0,           0;..  
           0,           0;..
           0,           0;..
           0,           0;..
           0,           0;..
           0,           0;..
           0,           0;..
           0,           0];
    // speed adaptation mechanism gains:
    okp=0.001; // proportional unit gain (p.u.)
    oki=0.05; // integral unit gain (p.u.)
case 3 then // reduced order integral unit PI observer matrices, formulas (36) - (37)
    G=[zeros(2,2);eye(2,2)];
    Ao=[A, G; zeros(2,4), -diag([0.1 0.1])];
    A3o= [A3, zeros(4,2); zeros(2,6)];
    Bo=[B; zeros(2,2)];
    Co1=[C, zeros(2,2)];
    K = [- 0.0693445,   0;..
           0,         - 0.0693445;..
           0.0007057,   0;..
           0,           0.0007057;..
           0.0123893,   0;..
           0,           0.0123893];
    K3= [  0,           0;..  
           0,           0;..
           0,           0;..
           0,           0;..
           0,           0;..
           0,           0];
    // speed adaptation mechanism gains:
    okp=0.001; // proportional unit gain (p.u.)
    oki=0.1; // integral unit gain (p.u.)
case 4 then // modified integral observer matrices, formulas (40)
    // matrices for gain selection process:
    Ao=[A, zeros(4,2); C, -diag([0.1 0.1])];
    A3o=[A3, zeros(4,2); zeros(2,6)];
    Bo=[B; zeros(2,2)];
    Co1=[zeros(2,4), eye(2,2)];
    // gain matrices: 
    K = [- 0.0924908,   0;..
           0,         - 0.0924908;..
           0.0930692,   0;..
           0,           0.0930692;..
         - 0.2816995,   0;..
           0,         - 0.2816995];
    K3= [  0,           0;..  
           0,           0;..
           0,           0;..
           0,           0;..
           0,           0;..
           0,           0];
    // matrices for simulation and practical application:
    Ao=[Ao+K*Co1, -K; zeros(2,6), -diag([0.1 0.1])]; //  Ao = | Ao        0_((n+q) x q) |
    A3o=[A3o+K3*Co1, -K3; zeros(2,6), zeros(2,2)];   //       | 0_(q x (n+q))   - Omega |
    Bo=[Bo; zeros(2,2)];                             //    Bo = | Bo        |
    Co1=zeros(2,8);                                  //         | 0_(q x p) |
    K=[zeros(6,2); -eye(2,2)];                       //  K = | 0_((n+q) x q) |    Co = 0_(q x (n+2*q))
    K3=K;                                            //      | -1_q          |
    // CAUTION: In case of this observer the matrices for simulation are different
    //          than for gain selection. It comes from the fact that the additional 
    //          integrator is not a part of an inner feedback loop of the observer.
    //          Therefore it does not take part in gain selection process. However,
    //          it must be a part of a ready-to-run observer. Formulas applied in
    //          lines 156 - 161 are not given in the paper because it is focused 
    //          on gain selection requirements. 
    // speed adaptation mechanism gains:
    okp=0.001; // proportional unit gain (p.u.)
    oki=0.05; // integral unit gain (p.u.)
case 51 then // proportional observer with 1 additional integrator matrices, formulas (47) - (49)
    G=[0, 0; 0, 0; 0, 1; 1, 0];
    Ao=[A, G; zeros(2,6)];
    Ao=Ao+[zeros(4,4), zeros(4,2); zeros(2,4), -diag([0.1 0.1])];
    A3o=A3; A3o(6,6)=0;
    Bo=[B; zeros(2,2)];
    Co1=[C, zeros(2,2)];
    // gain matrices:
    K = [- 0.0784493,   0;..
           0,         - 0.0784493;..
           0.0278771,   0;..
           0,           0.0278771;..
         - 0.0056867,   0;..
           0,         - 0.0056867];
    K3= [  0,           0;..  
           0,           0;..
           0,           0;..
           0,           0;..
           0,           0;..
           0,           0];
    // speed adaptation mechanism gains:
    okp=0.001; // proportional unit gain (p.u.)
    oki=0.05; // integral unit gain (p.u.)
case 52 then // proportional observer with 2 additional integrators matrices, formulas (47) - (49)
    G=[0, 0; 0, 0; 0, 1; 1, 0];..
    Ao=[A, zeros(4,2), G;.. 
        zeros(2,8);..
        zeros(2,4), eye(2,2), zeros(2,2)];
    Ao=Ao+[zeros(4,4), zeros(4,4); zeros(4,4), -diag([0.1 0.1 0.05 0.05])];
    A3o=A3; A3o(8,8)=0;
    Bo=[B; zeros(4,2)];
    Co1=[C, zeros(2,4)];
    // gain matrices:
    K = [- 0.0455305,   0;..
           0,         - 0.0455305;..
           0.0688395,   0;..
           0,           0.0688395;..
         - 0.0008310,   0;..
           0,         - 0.0008310;..
           0.0066522,   0;..
           0,           0.0066522];
    K3= [  0,           0;..  
           0,           0;..
           0,           0;..
           0,           0;..
           0,           0;..
           0,           0;..
           0,           0;..
           0,           0];
    // speed adaptation mechanism gains:
    okp=0.001; // proportional unit gain (p.u.)
    oki=0.05; // integral unit gain (p.u.)
else
    error('Wrong observer chosen')
end

// speed adaptation mechanism of the observer:
OmEst=-1;  // -1 - observer estimates the speed (mechanism is on) 
           //  1 - observer measures the speed (mechanism is off)

// U/f = const profile of the power inverter:
UFprofilTvec=[0 0.6 1 1.5 4]; // time [s]
UFprofilFvec=[0.02 1 1 -1 -1]; // frequency and voltage in reference to Un i fn
UmaxLimUF=1; // max voltage cut-off limit
UminLimUF=0.05; // min voltage cut-off limit

// PWM inverter parameters
UF_VDC=400; // PWM inverter supply voltage [V]
UF_fsw=500; // PWM switching frequency[Hz] (the greater frequency, the slower simulation!)

// load torque:
MstepTime=0.75; // step time
MinitVal=0; // initial value [Nm]
MfinalVal=50; // final value [Nm]

// parameter variations of the motor:
dRs=1.05; dRr=1.1; dLs=1; dLr=1; dLm=1; // 1 - no variation
                                        // 1.2 - increase by 20%

// deviation of the reference voltage:
dUs=0.98; // 1 - no variation
          // 1.2 - increase by 20%

// disturbances in the phase currents measured by the observer:
iAoffset=0; iBoffset=0.01; iCoffset=0; // DC offset [A]
iNoise=0.1; // random noice amplitude [A]
diA=0.98; diB=1; diC=1.02; // deviation (1 - no deviation; 1.2 - increase by 20%)
iNoiseSeed=2113248; // initial value of the random number generator

// disturbances in the speed measured by the observer:
// (these settings have an effect only when OmEst=-1 - the observer measures the speed)
omegaOffset=5; // offset [rpm]
omegaNoise=4; // random noice amplitude [rpm]
domega=1.01; // deviation (1 - no deviation; 1.2 - increase by 20%)
omegaNoiseSeed=2113247; // initial value of the random number generator

// result data acqusition sampling period:
Tsample=0.0001; // the smaller value, the more points on resulting plots,
                // number of points = TotalTime / Tsample
// simulation total time:
TotalTime=2;

// zero-crossing detection:
Zcross=1; // 1 - on (simulation is slower but more accurate)
          // 0 - off




// SIMULATION -----------------------------------------------------------------

exec('sim functions.sci',-1); // activate the functions of the sim model
BufferSize=TotalTime/Tsample+1; // sim results buffer size
// auxilary variables of the sim model:
no=size(Ao,'r'); // the number of the state variables of the observer
Ino=eye(no,no); 
// import of the Xcos sim model:
chk=importXcosDiagram('sim model.zcos');
disp(' ');disp(' ');
if chk then disp('Xcos sim file imported successfully');
else error('Xcos sim file import failed'); end
scs_m.props.tf=TotalTime; // sim end time
// running sim in batch mode:
disp('Starting sim in batch mode...');
tic(); xcos_simulate(scs_m, 4); MeasuredTime=toc();
disp('Sim completed. Total time: '+..
     string(floor(MeasuredTime/60))+' min '+..
     string(MeasuredTime-floor(MeasuredTime/60)*60)+' sec');




// GRAPHS ---------------------------------------------------------------------

fig=figure(); clf(fig); fig.figure_name='Sim results ';
fig.background=-2; fig.figure_size=[1200,800]; fig.figure_position=[10,10];

subplot(3,2,1); 
plot(WaveObs.time,WaveObs.values(:,1),'r',.. // estimated omega
     WaveSil.time,WaveSil.values(:,13),'b'); // motor omega
plt1=gca(); plt1.grid=[1 1]; plt1.x_label.text='$t \rm{, s}$';
plt1.x_label.font_size=4;
plt1.y_label.text='$\textcolor{255,0,0} ■  \phantom{i}  \hat\omega '+..
                   '{\rm,\phantom{x}}'+..
                   '\textcolor{0,0,255} ■  \phantom{i}  \omega '+..
                   '{\rm,\phantom{x} [p.u.]}$';
plt1.y_label.font_size=4;

subplot(3,2,3)
plot(WaveObs.time,(WaveObs.values(:,2).^2+WaveObs.values(:,3).^2).^0.5,'r',.. // estimated psi_sx
     WaveSil.time,(WaveSil.values(:,9).^2+WaveSil.values(:,10).^2).^0.5,'b'); // motor psi_sx
plt2=gca(); plt2.grid=[1 1]; plt2.x_label.text='$t \rm{, s}$';
plt2.x_label.font_size=4;
plt2.y_label.text='$\textcolor{255,0,0} ■  \phantom{i}  \hat\psi {_\rm s} '+..
                   '{\rm,\phantom{x}}'+..
                   '\textcolor{0,0,255} ■  \phantom{i}  \psi {_\rm s} '+..
                   '{\rm,\phantom{x}\rm[p.u.]}$';
plt2.y_label.font_size=4;

subplot(3,2,5)
plot(WaveObs.time,(WaveObs.values(:,4).^2+WaveObs.values(:,5).^2).^0.5,'r',..  // estimated psi_rx
     WaveSil.time,(WaveSil.values(:,11).^2+WaveSil.values(:,12).^2).^0.5,'b'); // motor psi_rx
plt3=gca(); plt3.grid=[1 1]; plt3.x_label.text='$t \rm{, s}$';
plt3.x_label.font_size=4;
plt3.y_label.text='$\textcolor{255,0,0} ■  \phantom{i}  \hat\psi {_\rm r} '+..
                   '{\rm,\phantom{x}}'+..
                   '\textcolor{0,0,255} ■  \phantom{i}  \psi {_\rm r} '+..
                   '{\rm,\phantom{x} [p.u.]}$';
plt3.y_label.font_size=4;

subplot(3,2,2)
plot(WaveSil.time,WaveSil.values(:,1),'r',.. // usx_ref
     WaveSil.time,WaveSil.values(:,2),'b'); // usy_ref
plt4=gca();
plt4.grid=[1 1];
plt4.x_label.text='$t \rm{, s}$';
plt4.x_label.font_size=4;
plt4.y_label.text='$\textcolor{255,0,0} ■  \phantom{i}  u {_\rm sx \phantom{i} ref} '+..
                   '{\rm,\phantom{i}}'+..
                   '\textcolor{0,0,255} ■  \phantom{i}  u {_\rm sy \phantom{i} ref} '+..
                   '\rm,\phantom{i}\rm{[V]} \phantom{x}$';
plt4.y_label.font_size=4;

subplot(3,2,4)
plot(WaveSil.time,WaveSil.values(:,14),'r',.. // electromagnetic torque
     WaveSil.time,WaveSil.values(:,15),'b');  // load torque
plt5=gca();
plt5.grid=[1 1];
plt5.x_label.text='$t \rm{, s}$';
plt5.x_label.font_size=4;
plt5.y_label.text='$\textcolor{255,0,0} ■  \phantom{i}  t {_\rm e} '+..
                   '{\rm,\phantom{i}}'+..
                   '\textcolor{0,0,255} ■  \phantom{i}  t {_\rm load} '+..
                   '{\rm,\phantom{i}\rm[Nm]}$';
plt5.y_label.font_size=4;

subplot(3,2,6)
plot(WaveSil.time,WaveSil.values(:,6),'r',.. // isA
     WaveSil.time,WaveSil.values(:,7),'b',.. // isA
     WaveSil.time,WaveSil.values(:,8),'g');  // isB
plt6=gca();
plt6.grid=[1 1];
plt6.x_label.text='$t \rm{, s}$';
plt6.x_label.font_size=4;
plt6.y_label.text='$\textcolor{255,0,0} ■  \phantom{i}  i {_\rm sA} '+..
                   '{\rm,\phantom{i}}'+..
                   '\textcolor{0,0,255} ■  \phantom{i}  i {_\rm sB} '+..
                   '{\rm,\phantom{i}}'+..
                   '\textcolor{0,255,0} ■  \phantom{i}  i {_\rm sC} '+..
                   '{\rm,\phantom{i}\rm[A]}$';
plt6.y_label.font_size=4;


